//    USB interface functions of RDS USB software 
//
// =====================================================
//  Description
//    This file include initialisation interface functions
//       of RDS software using : AVR USB driver  by OBJECTIVE DEVELOPMENT Software GmbH 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
//=======================================
// Author : G.Laroche www.g.laroche.free.fr
//=======================================
// History:
// 24/06/07    V1.0
//       create with only PSName and Ping command 
// 17/07/07    V1.1
//      adding all others USB commmand
//=======================================


/* Includes */
#include "RDS_USBInterface.h"
#include "RDS_EepromMGt.h" 
#include "RDS.h" 
#include "RDS_ClockDriver.h"
#include "RDS_IOMgt.h"

/* include USB command for RDS application */
#include "rds_usb.h"


#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/io.h>
#include <string.h>

/* pgmspace.h is necessary to use char string in flash memory */
/* PSTR("string")  to direct flash using, else String are duplicate into RAM  !!!! */
#include <avr/pgmspace.h>   

/* local constantes definitions */
#define USBINT_SIZE_DATA_BUFFER  64
#define USBINT_SIZE_REPLY_BUFFER  8


static unsigned char UsbInterfaceDataBuff[USBINT_SIZE_DATA_BUFFER];
static unsigned char UsbInterfaceReplyBuff[USBINT_SIZE_REPLY_BUFFER];


/* extern variables defined in USB driver */
extern unsigned char  * usbMsgPtr;

/* function usbpoll() shall be called every 10msec */

/* extern variables defined in RDS driver */
extern unsigned char CurrentPSName[RDS_SIZE_PSNAME];        /* current PS Name */
extern unsigned char CurrentRTxt[RDS_SIZE_RADIOTEXT];       /* current RadioText ended by 0x0d '\r' , unused data should be set to \r for optimisation */
extern unsigned char CurrentTxtAB;                          /* current Text A/B , each change clear line text */
extern unsigned short CurrentTP;                            /* current TP */
extern unsigned short CurrentPTY;                           /* current PTY */
extern unsigned char CurrentTA;                             /* current TA */
extern unsigned char CurrentMS;                             /* current MS */
extern unsigned char CurrentDI;                             /* current DI */


/***************************************************/
/*    RS_USBIntefaceInit :                         */
/*    --------                                     */
/*      initialize USB interface and driver        */
/***************************************************/

void RS_USBIntefaceInit( void )
{
        usbInit();			/* init USB driver */

}

/***************************************************/
/*    usbFunctionSetup :                           */
/*    --------                                     */
/*      This function is called by usbProcessRx    */
/*    when data available from host                */
/*        data [1]   : command request             */
/*        data [2]   : first param lsb             */
/*        data [3]   : first param msb             */
/*        data [4]   : second param lsb            */
/*        data [5]   : second param msb            */
/*        data [6]   : not used                    */
/*        data [7]   : not used                    */

/***************************************************/

uint8_t usbFunctionSetup(uint8_t data[USB_INTERFACE_SIZE_DATA])
{
     uint8_t ReplyLenght;
     unsigned char IndexData, IndexBuffer;

	usbMsgPtr = UsbInterfaceReplyBuff;  /* set pointer USB to reply buffer */
    ReplyLenght=0; 

    /* test command resquest */
    switch( data[1])
    {
    	case RDS_USB_PING:
    	{   /* ping : return 2 parameter  */
    	    UsbInterfaceReplyBuff[0] = data[2];   /* return LSB data[2]*/
            UsbInterfaceReplyBuff[1] = data[3];   /* return MSB data[3] */ 
            ReplyLenght=2; 
    	}
    	break;
    	
    	case RDS_USB_RESET_EEPROM:
    	{   /* reset all EEPROM data  to default values */
            EEP_SetDefault(); 
    	}
    	break;

/* UPDATE RAM   command */
/*======================*/
    
    	case RDS_USB_UPDATE_PS_1:
    	case RDS_USB_STORE_PS_1:
        { 
             UsbInterfaceDataBuff[0] = data[3];  /* LSB = first char */
             UsbInterfaceDataBuff[1] = data[2];  /* MSB = second char */
             UsbInterfaceDataBuff[2] = data[5];  /* LSB = third char */
             UsbInterfaceDataBuff[3] = data[4];  /* MSB = 4 char */             
        }
        break;


    	case RDS_USB_UPDATE_PS_2:
    	case RDS_USB_STORE_PS_2:
    	{
            UsbInterfaceDataBuff[4] = data[3];
            UsbInterfaceDataBuff[5] = data[2];
            UsbInterfaceDataBuff[6] = data[5];
            UsbInterfaceDataBuff[7] = data[4];
            /* last data, copy to RDS PSNAME */
            for (IndexData=0; IndexData<RDS_SIZE_PSNAME;IndexData++ )
            {
               CurrentPSName[IndexData]=UsbInterfaceDataBuff[IndexData];
            }
            /* if STORE EEPROM comande, set PSNAME in EEPROM */
            if (data[1] == RDS_USB_STORE_PS_2 )
            {
                /* store PSNAME in EEPROM */
                EEP_set_PSName(CurrentPSName);               
            }
        }
    	break;
    	
        /* For Radiotext all data should be send , even if lenght is smaller than 64 bytes */
        /* in this case PC software must fill unused data with '\r' */    	
    	case RDS_USB_UPDATE_RT_0:
    	case RDS_USB_UPDATE_RT_1:
    	case RDS_USB_UPDATE_RT_2:
    	case RDS_USB_UPDATE_RT_3:
    	case RDS_USB_UPDATE_RT_4:
    	case RDS_USB_UPDATE_RT_5:
    	case RDS_USB_UPDATE_RT_6:
    	case RDS_USB_UPDATE_RT_7:
    	case RDS_USB_UPDATE_RT_8:
    	case RDS_USB_UPDATE_RT_9:
    	case RDS_USB_UPDATE_RT_A:
    	case RDS_USB_UPDATE_RT_B:
    	case RDS_USB_UPDATE_RT_C:
    	case RDS_USB_UPDATE_RT_D:
    	case RDS_USB_UPDATE_RT_E:
    	case RDS_USB_UPDATE_RT_F:
        {
            /* data[1] = bloc number with offset RDS_USB_UPDATE_RT_0 (0=first bloc)*/
           
            IndexBuffer=(data[1]-RDS_USB_UPDATE_RT_0)<<2;
            UsbInterfaceDataBuff[IndexBuffer++] = data[3];
            UsbInterfaceDataBuff[IndexBuffer++] = data[2];
            UsbInterfaceDataBuff[IndexBuffer++] = data[5];
            UsbInterfaceDataBuff[IndexBuffer++] = data[4];
            /* if last data copy in current data */
            if (data[1] ==  RDS_USB_UPDATE_RT_F) 
            {              
              for (IndexData=0; IndexData<RDS_SIZE_RADIOTEXT;IndexData++ )
              {
                CurrentRTxt[IndexData]=UsbInterfaceDataBuff[IndexData];
              }
            }
                   
        }
        break;

    	case RDS_USB_UPDATE_PTY:
        { 
             CurrentPTY = ((uint16_t)data[2])<<5; /* currentPTY must be shifted for time optimisation before settting */
        }
        break;
    	case RDS_USB_UPDATE_TP:
        { 
             CurrentTP = ((uint16_t)data[2])<<10;  /* currentTP must be shifted for time optimisation before settting */
        }
        break;
    	case RDS_USB_UPDATE_TA:
        { 
             CurrentTA = data[2];  /* no shift necessary for TA */
        }
        break;
    	case RDS_USB_UPDATE_MS:
        { 
             CurrentMS = data[2]<<3;  /* currentMS must be shifted for time optimisation before settting */
        }
        break;
    	case RDS_USB_UPDATE_DI:
        { 
             CurrentDI = data[2];  /* no shift for DI */
        }
        break;



/* STORE EEPROM command */
/*======================*/

        /* For Radiotext all data should be send , even if lenght is smaller than 64 bytes */
        /* in this case PC software must fill unused data with '\r' */    	
    	case RDS_USB_STORE_RT_0:
    	case RDS_USB_STORE_RT_1:
    	case RDS_USB_STORE_RT_2:
    	case RDS_USB_STORE_RT_3:
    	case RDS_USB_STORE_RT_4:
    	case RDS_USB_STORE_RT_5:
    	case RDS_USB_STORE_RT_6:
    	case RDS_USB_STORE_RT_7:
    	case RDS_USB_STORE_RT_8:
    	case RDS_USB_STORE_RT_9:
    	case RDS_USB_STORE_RT_A:
    	case RDS_USB_STORE_RT_B:
    	case RDS_USB_STORE_RT_C:
    	case RDS_USB_STORE_RT_D:
    	case RDS_USB_STORE_RT_E:
    	case RDS_USB_STORE_RT_F:
    	{
            /* data[1] = bloc number with offset RDS_USB_STORE_RT_0 (0=first bloc)*/
           
            IndexBuffer=(data[1]-RDS_USB_STORE_RT_0)<<2;
            UsbInterfaceDataBuff[IndexBuffer++] = data[3];
            UsbInterfaceDataBuff[IndexBuffer++] = data[2];
            UsbInterfaceDataBuff[IndexBuffer++] = data[5];
            UsbInterfaceDataBuff[IndexBuffer++] = data[4];
            /* if last data copy in current data */
            if (data[1] ==  RDS_USB_STORE_RT_F) 
            {              
              for (IndexData=0; IndexData<RDS_SIZE_RADIOTEXT;IndexData++ )
              {
                CurrentRTxt[IndexData]=UsbInterfaceDataBuff[IndexData];
              }
            }
              /* store PSNAME in EEPROM */
              EEP_set_RadioText(CurrentRTxt);               
        }
    	break;
    	case RDS_USB_STORE_PTY:
        { 
             EEP_set_PTY(data[2]);  /* LSB = first char */
        }
        break;
    	case RDS_USB_STORE_TP:
        { 
             EEP_set_TP(data[2]);  /* LSB = first char */
        }
        break;
    	case RDS_USB_STORE_TA:
        { 
             EEP_set_TA(data[2]);  /* LSB = first char */
        }
        break;
    	case RDS_USB_STORE_MS:
        { 
             EEP_set_MS(data[2]);  /* LSB = first char */
        }
        break;
    	case RDS_USB_STORE_DI:
        { 
             EEP_set_DI(data[2]);  /* LSB = first char */
        }
        break;
    	case RDS_USB_STORE_PICODE:
        { 
             EEP_set_PICode((uint16_t)data[2]+(((uint16_t)(data[3]))<<8));  /* MSB = second char :data[3]*/
        }
        break;
    	case RDS_USB_STORE_NA:
        { 
             EEP_set_NbAF(data[2]);  /* LSB = first char */
        }
        break;
    	case RDS_USB_STORE_AF:
        { 
             EEP_set_AF(data[2],data[4]);  /* 1st param = freq, second=index */
        }
        break;
      	case RDS_USB_CONFIG_2A:
        { 
             /* for this version, only 2A group rate available */
             EEP_set_GroupRate(data[2],(RDS_GROUP_TYPE_2A & RDS_GROUP_TYPE_SELECT),(RDS_GROUP_TYPE_2A>>RDS_GROUP_TYPE_SHIFT));
        }
        break;
    	case RDS_USB_CONFIG_STEREO:
        { 
             EEP_set_StereoMode(data[2]);
        }
        break;
  
    	
/*  READ  RAM   command */
/*======================*/
    	
    	case RDS_USB_READ_PS_1:
        { 
    	    UsbInterfaceReplyBuff[0] = CurrentPSName[0];
            UsbInterfaceReplyBuff[1] = CurrentPSName[1]; 
    	    UsbInterfaceReplyBuff[2] = CurrentPSName[2];
            UsbInterfaceReplyBuff[3] = CurrentPSName[3]; 
            ReplyLenght=4; 
        }
        break;
    	case RDS_USB_READ_PS_2:
        { 
    	    UsbInterfaceReplyBuff[0] = CurrentPSName[4];
            UsbInterfaceReplyBuff[1] = CurrentPSName[5]; 
    	    UsbInterfaceReplyBuff[2] = CurrentPSName[6];
            UsbInterfaceReplyBuff[3] = CurrentPSName[7]; 
            ReplyLenght=4; 
        }
        break;



    	case RDS_USB_READ_RT_0:
    	case RDS_USB_READ_RT_1:
    	case RDS_USB_READ_RT_2:
    	case RDS_USB_READ_RT_3:
    	case RDS_USB_READ_RT_4:
    	case RDS_USB_READ_RT_5:
    	case RDS_USB_READ_RT_6:
    	case RDS_USB_READ_RT_7:
    	case RDS_USB_READ_RT_8:
    	case RDS_USB_READ_RT_9:
    	case RDS_USB_READ_RT_A:
    	case RDS_USB_READ_RT_B:
    	case RDS_USB_READ_RT_C:
    	case RDS_USB_READ_RT_D:
    	case RDS_USB_READ_RT_E:
    	case RDS_USB_READ_RT_F:
        {
            /* data[1] = bloc number with offset RDS_USB_UPDATE_RT_0 (0=first bloc)*/       
            IndexBuffer=(data[1]-RDS_USB_READ_RT_0)<<2;
            UsbInterfaceReplyBuff[0] = CurrentRTxt[IndexBuffer];
            UsbInterfaceReplyBuff[1] = CurrentRTxt[IndexBuffer+1];
            UsbInterfaceReplyBuff[2] = CurrentRTxt[IndexBuffer+2];
            UsbInterfaceReplyBuff[3] = CurrentRTxt[IndexBuffer+3];
            ReplyLenght=4;                                    
        }
        break;
    	case RDS_USB_READ_PTY:
        { 
    	    UsbInterfaceReplyBuff[0] = CurrentPTY >>5;  /* currentPTY is previously shifted for time optimisation */
            ReplyLenght=1; 
        }
        break;
    	case RDS_USB_READ_TP:
        { 
    	    UsbInterfaceReplyBuff[0] = CurrentTP >>10;/* currentTP is previously shifted for time optimisation */
            ReplyLenght=1; 
        }
        break;
    	case RDS_USB_READ_TA:
        { 
    	    UsbInterfaceReplyBuff[0] = CurrentTA;
            ReplyLenght=1; 
        }
        break;
    	case RDS_USB_READ_MS:
        { 
    	    UsbInterfaceReplyBuff[0] = CurrentMS >>3; /* currentMS is previously shifted for time optimisation */
            ReplyLenght=1; 
        }
        break;
    	case RDS_USB_READ_DI:
        { 
    	    UsbInterfaceReplyBuff[0] = CurrentDI;
            ReplyLenght=1; 
        }
        break;


/*  READ EEPROM command */
/*======================*/

    	case RDS_USB_READ_E2P_PS_1:
        { 
    	    UsbInterfaceReplyBuff[0] = EEP_get_PSName(0);
            UsbInterfaceReplyBuff[1] = EEP_get_PSName(1); 
    	    UsbInterfaceReplyBuff[2] = EEP_get_PSName(2);
            UsbInterfaceReplyBuff[3] = EEP_get_PSName(3); 
            ReplyLenght=4; 
        }
        break;
    	case RDS_USB_READ_E2P_PS_2:
        { 
    	    UsbInterfaceReplyBuff[0] = EEP_get_PSName(4);
            UsbInterfaceReplyBuff[1] = EEP_get_PSName(5); 
    	    UsbInterfaceReplyBuff[2] = EEP_get_PSName(6);
            UsbInterfaceReplyBuff[3] = EEP_get_PSName(7); 
            ReplyLenght=4; 
        }
        break;

    	case RDS_USB_READ_E2P_RT_0:
    	case RDS_USB_READ_E2P_RT_1:
    	case RDS_USB_READ_E2P_RT_2:
    	case RDS_USB_READ_E2P_RT_3:
    	case RDS_USB_READ_E2P_RT_4:
    	case RDS_USB_READ_E2P_RT_5:
    	case RDS_USB_READ_E2P_RT_6:
    	case RDS_USB_READ_E2P_RT_7:
    	case RDS_USB_READ_E2P_RT_8:
    	case RDS_USB_READ_E2P_RT_9:
    	case RDS_USB_READ_E2P_RT_A:
    	case RDS_USB_READ_E2P_RT_B:
    	case RDS_USB_READ_E2P_RT_C:
    	case RDS_USB_READ_E2P_RT_D:
    	case RDS_USB_READ_E2P_RT_E:
    	case RDS_USB_READ_E2P_RT_F:
        {
            /* data[1] = bloc number with offset RDS_USB_UPDATE_RT_0 (0=first bloc)*/       
            IndexBuffer=(data[1]-RDS_USB_READ_E2P_RT_0)<<2;
            UsbInterfaceReplyBuff[0] = EEP_get_Radiotext(IndexBuffer);
            UsbInterfaceReplyBuff[1] = EEP_get_Radiotext(IndexBuffer+1);
            UsbInterfaceReplyBuff[2] = EEP_get_Radiotext(IndexBuffer+2);
            UsbInterfaceReplyBuff[3] = EEP_get_Radiotext(IndexBuffer+3);
            ReplyLenght=4;                 
        }
        break;
    	case RDS_USB_READ_E2P_PTY:
        { 
    	    UsbInterfaceReplyBuff[0] = EEP_get_PTY();
            ReplyLenght=1; 
        }
        break;
    	case RDS_USB_READ_E2P_TP:
        { 
    	    UsbInterfaceReplyBuff[0] = EEP_get_TP();
            ReplyLenght=1; 
        }
        break;
    	case RDS_USB_READ_E2P_TA:
        { 
    	    UsbInterfaceReplyBuff[0] = EEP_get_TA();
            ReplyLenght=1; 
        }
        break;
    	case RDS_USB_READ_E2P_MS:
        { 
    	    UsbInterfaceReplyBuff[0] = EEP_get_MS();
            ReplyLenght=1; 
        }
        break;
    	case RDS_USB_READ_E2P_DI:
        { 
    	    UsbInterfaceReplyBuff[0] = EEP_get_DI();
            ReplyLenght=1; 
        }
        break;
    	case RDS_USB_READ_E2P_PICODE:
        { 
             UsbInterfaceReplyBuff[0] = (char)(EEP_get_PICode() >>8); /* return MSB first */
             UsbInterfaceReplyBuff[1] = (char)(EEP_get_PICode() & 0xFF);
             ReplyLenght=2; 
        }
        break;
    	case RDS_USB_READ_E2P_NA:
        { 
             UsbInterfaceReplyBuff[0] = EEP_get_NbAF();
             ReplyLenght=1; 
        }
        break;
    	case RDS_USB_READ_E2P_AF:
        { 
             UsbInterfaceReplyBuff[0] = EEP_get_AF(data[2]);
             ReplyLenght=1; 
        }
        break;
    	case RDS_USB_READ_CONFIG_2A:
        { 
             /* for this version, only 2A group rate available */
             UsbInterfaceReplyBuff[0] = EEP_get_GroupRate((RDS_GROUP_TYPE_2A & RDS_GROUP_TYPE_SELECT),(RDS_GROUP_TYPE_2A>>RDS_GROUP_TYPE_SHIFT));
             ReplyLenght=1; 
        }
        break;
    	case RDS_USB_READ_CONFIG_STEREO:
        { 
             UsbInterfaceReplyBuff[0] = EEP_get_StereoMode();
             ReplyLenght=1; 
        }
        break;

//    	default:
    
    }
    
    
    return (ReplyLenght);
}



